@@ -1,19 +1,20 @@
 #!/usr/bin/env python3
+#
+# This file is part of pyspex
+#
+# https://github.com/rmvanhees/pyspex.git
+#
+# Copyright (c) 2019-2022 SRON - Netherlands Institute for Space Research
+#    All Rights Reserved
+#
+# License:  BSD-3-Clause
 """
-This file is part of pyspex
+Python implementation SPEXone instrument simulator output to L1A.
 
-https://github.com/rmvanhees/pyspex.git
+Environment::
 
-Python implementation SPEXone instrument simulator output to L1A
+   CKD_DIR:  directory with SPEXone CKD, default is CWD.
 
-Environment
------------
-CKD_DIR :  directory with SPEXone CKD, default is CWD
-
-Copyright (c) 2019-2021 SRON - Netherlands Institute for Space Research
-   All Rights Reserved
-
-License:  BSD-3-Clause
 """
 import argparse
 from datetime import datetime, timezone
@@ -25,7 +26,7 @@ import numpy as np
 import xarray as xr
 
 from pyspex import spx_product
-from pyspex.lib.tmtc_def import tmtc_def
+from pyspex.lib.tmtc_def import tmtc_dtype
 from pyspex.tif_io import TIFio
 from pyspex.lv1_io import L1Aio
 from pyspex.lv1_gse import LV1gse
@@ -89,6 +90,10 @@ def get_stimulus(hdr):
 def main():
     """
     main program to illustate the creation of a L1A calibration product
+
+    Environment
+    -----------
+    CKD_DIR :  directory with SPEXone CKD, default is CWD
     """
     parser = argparse.ArgumentParser(
         description=('create SPEXone L1A product from instrument simulations'))
@@ -188,9 +193,10 @@ def main():
     secnds = (1 + np.arange(n_images, dtype=float)) * intg_time
     img_sec = (offs + secnds // 1).astype('u4')
     img_subsec = np.round((secnds % 1) * 2**16).astype('u2')
+    img_time = secnds
 
-    nom_hk = np.zeros(n_images, dtype=np.dtype(tmtc_def(0x320)))
-    sci_hk = np.zeros(n_images, dtype=np.dtype(tmtc_def(0x350)))
+    nom_hk = np.zeros(n_images, dtype=tmtc_dtype(0x320))
+    sci_hk = np.zeros(n_images, dtype=tmtc_dtype(0x350))
     if table_id == 0:
         sci_hk['REG_FULL_FRAME'] = 1
         sci_hk['REG_CMV_OUTPUTMODE'] = 3
@@ -205,15 +211,17 @@ def main():
     sci_hk['REG_NCOADDFRAMES'] = int(hdr['Co-additions'])
 
     # Generate L1A product
-    with L1Aio(prod_name, dims=dims) as l1a:
+    with L1Aio(prod_name, dims=dims, ref_date=utc_start.date()) as l1a:
         # write image data, detector telemetry and image attributes
         l1a.fill_science(images.reshape(n_images, -1), sci_hk,
                          np.arange(n_images))
-        l1a.fill_time(img_sec, img_subsec, group='image_attributes')
+        l1a.set_dset('/image_attributes/icu_time_sec', img_sec)
+        l1a.set_dset('/image_attributes/icu_time_subsec', img_subsec)
+        l1a.set_dset('/image_attributes/image_time', img_time)
 
         # Engineering data
         l1a.fill_nomhk(nom_hk)
-        l1a.fill_time(img_sec, img_subsec, group='engineering_data')
+        l1a.set_dset('/engineering_data/HK_tlm_time', img_time)
 
         # Global attributes
         l1a.fill_global_attrs(inflight=False)
