@@ -1,37 +1,40 @@
+#
+# This file is part of pyspex
+#
+# https://github.com/rmvanhees/pyspex.git
+#
+# Copyright (c) 2019-2022 SRON - Netherlands Institute for Space Research
+#    All Rights Reserved
+#
+# License:  BSD-3-Clause
 """
-This file is part of pyspex
-
-https://github.com/rmvanhees/pyspex.git
-
-Defines global attributes for SPEXone L1 products
-
-Copyright (c) 2020-2021 SRON - Netherlands Institute for Space Research
-   All Rights Reserved
-
-License:  BSD-3-Clause
+Provides the common global-attributes for SPEXone Level-1 products.
 """
+__all__ = ['attrs_def']
+
 from datetime import datetime, timezone
 
 from pyspex import version
 
 
-# - local functions --------------------------------
+# - main functions --------------------------------
 def attrs_def(level: str, inflight=True, origin=None) -> dict:
     """
-    Defines all global attributes for SPEXone Level-1 products
+    Defines all global attributes for SPEXone Level-1 products.
 
     Parameters
     ----------
     level : str
-      Product processing level 'L1A', 'L1B' or 'L1C'
+       Product processing level 'L1A', 'L1B' or 'L1C'
     inflight : bool
-      Flag for in-flight or on-ground products
+       Flag for in-flight or on-ground products
     origin : str
-      Product origin: 'SRON' or 'NASA'
+       Product origin: 'SRON' or 'NASA'
 
     Returns
     -------
     dict
+       Global attributes for a Level-1A product
     """
     if origin is None:
         origin = 'NASA' if inflight else 'SRON'
@@ -83,10 +86,10 @@ def attrs_def(level: str, inflight=True, origin=None) -> dict:
         res['title'] = f"SPEXone Level-{level[1:]} data"
         res['instrument'] = "SPEXone"
         res['institution'] = "SRON Netherlands Institute for Space Research"
-        res['creator_name'] = "SRON/Earth Science"
+        res['creator_name'] = "SRON/Earth"
         res['creator_email'] = "SPEXone-MPC@sron.nl"
         res['creator_url'] = "https://www.sron.nl/missions-earth/pace-spexone"
-        res['publisher_name'] = "SRON/Earth Science"
+        res['publisher_name'] = "SRON/Earth"
         res['publisher_email'] = "SPEXone-MPC@sron.nl"
         res['publisher_url'] = "https://www.sron.nl/missions-earth/pace-spexone"
 
