@@ -1,25 +1,30 @@
+#
+# This file is part of pyspex
+#
+# https://github.com/rmvanhees/pyspex.git
+#
+# Copyright (c) 2019-2022 SRON - Netherlands Institute for Space Research
+#    All Rights Reserved
+#
+# License:  BSD-3-Clause
 """
-This file is part of pyspex
-
-https://github.com/rmvanhees/pyspex.git
-
-Python implementation of the PACE SPEX Level-1B product
-
-Copyright (c) 2020-2021 SRON - Netherlands Institute for Space Research
-   All Rights Reserved
-
-License:  BSD-3-Clause
+Defines the format of a SPEXone Level-1B product.
 """
+__all__ = ['init_l1b']
+
+import datetime
+
 from netCDF4 import Dataset
 
 # - global parameters ------------------------------
-
+ORBIT_DURATION = 5904  # seconds
 
 # - local functions --------------------------------
 
 
 # - main function ----------------------------------
-def init_l1b(l1b_flname: str, ref_date, dims: dict):
+# pylint: disable=too-many-statements
+def init_l1b(l1b_flname: str, ref_date: datetime.date, dims: dict) -> None:
     """
     Create an empty PACE SPEX Level-1B product
 
@@ -27,15 +32,16 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
     ----------
     l1b_flname : string
        Name of Level-1B product
-    ref_date : datetime.datetime
-       Reference date for image time
+    ref_date : datetime.date
+       Date of the first detector image
     dims :   dictionary
        Provide length of the Level-1B dimensions
-       Default values:
-          bins_along_track :  400
-          spatial_samples_per_image :  200
-          intensity_bands_per_view :  50
-          polarization_bands_per_view :  50
+       Default values::
+
+          bins_along_track: 400
+          spatial_samples_per_image: 200
+          intensity_bands_per_view: 50
+          polarization_bands_per_view: 50
 
     """
     # check function parameters
@@ -72,14 +78,18 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
     sgrp = rootgrp.createGroup('BIN_ATTRIBUTES')
     chunksizes = None if n_bins_along is not None else (512,)
     dset = sgrp.createVariable('image_time', 'f8', ('bins_along_track',),
-                               chunksizes=chunksizes)
-    dset.long_name = 'image time (seconds of day)'
-    dset.valid_min = 0
-    dset.valid_max = 86400.999999
+                               chunksizes=chunksizes, fill_value=-32767)
+    dset.long_name = "Image time"
+    dset.description = "Integration start time in seconds of day"
     if ref_date is None:
-        dset.units = 'seconds'
+        dset.units = "second since midnight"
     else:
-        dset.units = f'seconds since {ref_date.isoformat(sep=" ")}'
+        dset.units = f"seconds since {ref_date.isoformat()} 00:00:00"
+        dset.year = f"{ref_date.year}"
+        dset.month = f"{ref_date.month}"
+        dset.day = f"{ref_date.day}"
+    dset.valid_min = 0
+    dset.valid_max = 86400 + ORBIT_DURATION
 
     # -------------------------
     sgrp = rootgrp.createGroup('GEOLOCATION_DATA')
@@ -91,7 +101,7 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
                                chunksizes=chunksizes)
     dset.standard_name = 'altitude'
     dset.long_name = "height above mean sea level"
-    dset.units = "meters"
+    dset.units = "m"
     dset.positive = "up"
     dset.axis = "Z"
     dset = sgrp.createVariable('latitude', 'f4',
@@ -117,25 +127,25 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
                                 'spatial_samples_per_image'),
                                chunksizes=chunksizes)
     dset.long_name = 'sensor azimuth angle'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('sensor_zenith', 'f4',
                                ('bins_along_track',
                                 'spatial_samples_per_image'),
                                chunksizes=chunksizes)
     dset.long_name = 'sensor zenith angle'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('solar_azimuth', 'f4',
                                ('bins_along_track',
                                 'spatial_samples_per_image'),
                                chunksizes=chunksizes)
     dset.long_name = 'solar azimuth angle'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('solar_zenith', 'f4',
                                ('bins_along_track',
                                 'spatial_samples_per_image'),
                                chunksizes=chunksizes)
     dset.long_name = 'solar zenith angle'
-    dset.units = 'degrees'
+    dset.units = 'degree'
 
     # -------------------------
     sgrp = rootgrp.createGroup('OBSERVATION_DATA')
@@ -146,13 +156,13 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
                                 'intensity_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'I Stokes vector component'
-    dset.units = 'W.m-2.sr-1.um-1'
+    dset.units = 'W/(m^2.sr.um)'
     dset = sgrp.createVariable('I_noise', 'f4',
                                ('bins_along_track', 'spatial_samples_per_image',
                                 'intensity_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'random noise of I'
-    dset.units = 'W.m-2.sr-1.um-1'
+    dset.units = 'W/(m^2.sr.um)'
 
     chunksizes = None if n_bins_along is not None else \
         (64, n_spatial_samples, n_polar_bands)
@@ -185,13 +195,13 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
                                 'polarization_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'angle of linear polarization'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('AoLP_noise', 'f4',
                                ('bins_along_track', 'spatial_samples_per_image',
                                 'polarization_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'random noise of AoLP'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('DoLP', 'f4',
                                ('bins_along_track', 'spatial_samples_per_image',
                                 'polarization_bands_per_view'),
@@ -215,7 +225,7 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
     dset.comment = "Contains indices to each viewport for all spatial samples."
     dset = sgrp.createVariable('view_angles', 'f4', ('number_of_views',))
     dset.long_name = 'along track view zenith angles at sensor'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset.comment = ('view_angles is defined at the sensor, as it provides'
                     ' a swath independent value at TOA.')
     dset = sgrp.createVariable('intensity_wavelengths', 'f4',
@@ -229,7 +239,7 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
     dset = sgrp.createVariable('intensity_F0', 'f4',
                                ('number_of_views', 'intensity_bands_per_view'))
     dset.long_name = 'Solar irradiance on intensity wavelength grid'
-    dset.units = 'W.m-2'
+    dset.units = 'W/m^2'
     dset.comment = ('Spectral response function convolved mean solar flux'
                     ' at each intensity band and view.')
     dset = sgrp.createVariable('polarization_wavelengths', 'f4',
@@ -246,16 +256,8 @@ def init_l1b(l1b_flname: str, ref_date, dims: dict):
                                ('number_of_views',
                                 'polarization_bands_per_view'))
     dset.long_name = 'Solar irradiance on polarization wavelength grid'
-    dset.units = 'W.m-2'
+    dset.units = 'W/m^2'
     dset.comment = ('Spectral response function convolved mean solar flux'
                     ' at each polarization band and view.')
 
     return rootgrp
-
-
-# --------------------------------------------------
-if __name__ == '__main__':
-    fid = init_l1b('PACE_SPEX.20230115T123456.L1B.2.5km.V01.nc',
-                   ref_date=None, dims={})
-    fid.fill_global_attrs(orbit=12345, bin_size='2.5km')
-    fid.close()
