@@ -1,27 +1,32 @@
+#
+# This file is part of pyspex
+#
+# https://github.com/rmvanhees/pyspex.git
+#
+# Copyright (c) 2019-2022 SRON - Netherlands Institute for Space Research
+#    All Rights Reserved
+#
+# License:  BSD-3-Clause
 """
-This file is part of pyspex
-
-https://github.com/rmvanhees/pyspex.git
-
-Python implementation of the PACE SPEX Level-1C product
-
-Copyright (c) 2020-2021 SRON - Netherlands Institute for Space Research
-   All Rights Reserved
-
-License:  BSD-3-Clause
+Defines the format of a SPEXone Level-1C product.
 """
+__all__ = ['init_l1c']
+
+import datetime
+
 from netCDF4 import Dataset
 import numpy as np
 
 
 # - global parameters ------------------------------
-
+ORBIT_DURATION = 5904  # seconds
 
 # - local functions --------------------------------
 
 
 # - main function ----------------------------------
-def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
+# pylint: disable=too-many-statements
+def init_l1c(l1c_flname: str, ref_date: datetime.date, dims: dict) -> None:
     """
     Create an empty PACE SPEX Level-1C product
 
@@ -29,19 +34,16 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
     ----------
     l1c_flname : string
        Name of Level-1C product
-    ref_date : datetime.datetime
-       Reference date for image time
+    ref_date : datetime.date
+       Date of the first detector image
     dims :   dictionary
-       Provide length of the Level-1B dimensions
-       Default values:
-          bins_along_track :  400
-          bins_across_track :  20
-          intensity_bands_per_view :  50
-          polarization_bands_per_view :  50
+       Provide length of the Level-1C dimensions
+       Default values::
 
-    Notes
-    -----
-    ...
+          bins_along_track: 400
+          spatial_samples_per_image: 200
+          intensity_bands_per_view: 50
+          polarization_bands_per_view: 50
     """
     # check function parameters
     if not isinstance(dims, dict):
@@ -78,21 +80,28 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
     chunksizes = None if n_bins_along is not None else (512,)
     dset = sgrp.createVariable('nadir_view_time', 'f8',
                                ('bins_along_track',), chunksizes=chunksizes)
-    dset.long_name = 'time bin was viewed at nadir view'
+    dset.long_name = 'Nadir view time'
+    dset.description = "time when bin was viewed at nadir"
     dset.valid_min = 0
     dset.valid_max = 86400.999999
     if ref_date is None:
-        dset.units = 'seconds'
+        dset.units = "second since midnight"
     else:
-        dset.units = f'seconds since {ref_date.isoformat(sep=" ")}'
+        dset.units = f"seconds since {ref_date.isoformat()} 00:00:00"
+        dset.year = f"{ref_date.year}"
+        dset.month = f"{ref_date.month}"
+        dset.day = f"{ref_date.day}"
+    dset.valid_min = 0
+    dset.valid_max = 86400 + ORBIT_DURATION
     chunksizes = None if n_bins_along is not None else (512, n_bins_across)
     dset = sgrp.createVariable('view_time_offsets', 'f8',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views'), chunksizes=chunksizes)
-    dset.long_name = 'time offsets of views from nadir view'
+    dset.long_name = "time offsets of views"
+    dset.description = "offset of views wrt nadir view"
     dset.valid_min = -200
     dset.valid_max = 200
-    dset.units = 'seconds'
+    dset.units = 'second'
 
     # -------------------------
     sgrp = rootgrp.createGroup('GEOLOCATION_DATA')
@@ -101,14 +110,14 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
                                chunksizes=chunksizes)
     dset.standard_name = 'altitude'
     dset.long_name = "altitude at bin locations"
-    dset.units = "meters"
+    dset.units = "m"
     dset.positive = "up"
     dset.axis = "Z"
     dset = sgrp.createVariable('altitude_variability', 'f4',
                                ('bins_along_track', 'bins_across_track'),
                                chunksizes=chunksizes)
     dset.long_name = 'RMS variability of altitude at bin locations'
-    dset.units = 'meters'
+    dset.units = 'm'
     dset = sgrp.createVariable('latitude', 'f4',
                                ('bins_along_track', 'bins_across_track'),
                                chunksizes=chunksizes)
@@ -132,23 +141,23 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
                                 'number_of_views'), chunksizes=chunksizes)
     dset.long_name = 'sensor azimuth angle at bin locations'
     dset.comment = 'clockwise from north'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('sensor_zenith', 'f4',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views'), chunksizes=chunksizes)
     dset.long_name = 'sensor zenith angle at bin locations'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('solar_azimuth', 'f4',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views'), chunksizes=chunksizes)
     dset.long_name = 'solar azimuth angle at bin locations'
     dset.comment = 'clockwise from north'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('solar_zenith', 'f4',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views'), chunksizes=chunksizes)
     dset.long_name = 'solar zenith angle at bin locations'
-    dset.units = 'degrees'
+    dset.units = 'degree'
 
     # -------------------------
     sgrp = rootgrp.createGroup('OBSERVATION_DATA')
@@ -168,7 +177,7 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
                                 'number_of_views', 'pol_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'angle of linear polarization'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset = sgrp.createVariable('AoLP_noise', 'f4',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views', 'pol_bands_per_view'),
@@ -194,13 +203,13 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
                                 'number_of_views', 'intensity_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'I Stokes vector component'
-    dset.units = 'W.m-2.sr-1.um-1'
+    dset.units = 'W/(m^2.sr.um)'
     dset = sgrp.createVariable('I_noise', 'f4',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views', 'intensity_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'random noise of I in bin'
-    dset.units = 'W.m-2.sr-1.um-1'
+    dset.units = 'W/(m^2.sr.um)'
     chunksizes = None if n_bins_along is not None \
         else (256, n_bins_across, n_views, n_polar_bands)
     dset = sgrp.createVariable('I_polsample', 'f4',
@@ -209,13 +218,13 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
                                chunksizes=chunksizes)
     dset.long_name = \
         'I Stokes vector component at polarization band spectal sampling'
-    dset.units = 'W.m-2.sr-1.um-1'
+    dset.units = 'W/(m^2.sr.um)'
     dset = sgrp.createVariable('I_polsample_noise', 'f4',
                                ('bins_along_track', 'bins_across_track',
                                 'number_of_views', 'pol_bands_per_view'),
                                chunksizes=chunksizes)
     dset.long_name = 'random noise of I_polsample in bin'
-    dset.units = 'W.m-2.sr-1.um-1'
+    dset.units = 'W/(m^2.sr.um)'
     chunksizes = None if n_bins_along is not None \
         else (32, n_bins_across, n_views, n_intens_bands)
     dset = sgrp.createVariable('QC', 'f4',
@@ -288,7 +297,7 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
     dset = sgrp.createVariable('intensity_F0', 'f4',
                                ('number_of_views', 'intensity_bands_per_view'))
     dset.long_name = 'intensity band solar irradiance'
-    dset.units = 'W.m-2'
+    dset.units = 'W/m^2'
     dset = sgrp.createVariable('intensity_wavelengths', 'f4',
                                ('number_of_views', 'intensity_bands_per_view'))
     dset.long_name = 'intensity field center wavelengths at each view'
@@ -300,7 +309,7 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
     dset = sgrp.createVariable('polarization_F0', 'f4',
                                ('number_of_views', 'pol_bands_per_view'))
     dset.long_name = 'polarization band solar irradiance'
-    dset.units = 'W.m-2'
+    dset.units = 'W/m^2'
     dset.comment = ('Spectral response function convolved mean solar flux'
                     ' at each polarization band and view.')
     dset = sgrp.createVariable('polarization_wavelengths', 'f4',
@@ -311,16 +320,8 @@ def init_l1c(l1c_flname: str, ref_date, dims: dict) -> None:
                     ' at each intensity band and view.')
     dset = sgrp.createVariable('view_angles', 'f4', ('number_of_views',))
     dset.long_name = 'along-track view angles for sensor'
-    dset.units = 'degrees'
+    dset.units = 'degree'
     dset.comment = ('view_angles is defined at the sensor, as it provides'
                     ' a swath independent value at TOA.')
 
     return rootgrp
-
-
-# --------------------------------------------------
-if __name__ == '__main__':
-    fid = init_l1c('PACE_SPEX.20230115T123456.L1C.5km.V01.nc',
-                   ref_date=None, dims={})
-    fid.fill_global_attrs(orbit=12345, bin_size='5km')
-    fid.close()
