@@ -1,38 +1,37 @@
+#
+# This file is part of pyspex
+#
+# https://github.com/rmvanhees/pyspex.git
+#
+# Copyright (c) 2019-2022 SRON - Netherlands Institute for Space Research
+#    All Rights Reserved
+#
+# License:  BSD-3-Clause
 """
-This file is part of pyspex
+Defines the SPEXone telemetry packets as numpy data-types.
 
-https://github.com/rmvanhees/pyspex.git
-
-Defines SPEXone telemetry packet structures as a list of tuples to create
-Numpy structured datatypes
-
-Copyright (c) 2020-2021 SRON - Netherlands Institute for Space Research
-   All Rights Reserved
-
-License:  BSD-3-Clause
+References
+----------
+* SPX1-TN-005 Telemetry and Telecommand Handbook, issue 14, date 15-Mar-2021
 """
+__all__ = ['tmtc_dtype']
+
 import numpy as np
 
 
-def tmtc_def(apid):
-    """
-    Returns definition of a SPEXone telemetry packet structure
+def __tmtc_def(apid) -> list:
+    """Returns SPEXone telemetry packet structure as a list of tuples.
 
     Parameters
     ----------
     apid : int
-       Telemetry APID range between 0x320 and 0x350 for SPEXone.
-       Implemented APIDs are 0x350: Science TM, 0x320: NomHK, 0x322: DemHK
+       SPEXone telemetry APID.
+       Implemented APIDs: 0x350 (Science), 0x320 (NomHK) and 0x322 (DemHK).
 
     Returns
     -------
-    list of tuples to create a Numpy structured datatype
-
-    Examples
-    --------
-    >>> import numpy as np
-    >>> from pyspex.lib.tmtc_def import tmtc_def
-    >>> mps_dtype = np.dtype(tmtc_def(0x350))
+    list of tuples
+       Definition of a numpy structured datatype.
     """
     if apid == 0x350:                           # *** Science TM ***
         return [                                # offs  start in packet
@@ -461,7 +460,25 @@ def tmtc_def(apid):
 
 
 def tmtc_dtype(apid):
+    """Obtain SPEXone telemetry packet definition.
+
+    Parameters
+    ----------
+    apid : int
+       SPEXone telemetry APID.
+       Implemented APIDs: 0x350 (Science), 0x320 (NomHK) and 0x322 (DemHK).
+
+    Returns
+    -------
+    numpy.dtype
+       Definition of Spexone telemetry packet.
+
+    Examples
+    --------
+
+    >>> import numpy as np
+    >>> from pyspex.lib.tmtc_def import tmtc_dtype
+    >>> mps_dtype = tmtc_dtype(0x350)
+
     """
-    Returns Telemetry packet structure as a numpy data-type
-    """
-    return np.dtype(tmtc_def(apid))
+    return np.dtype(__tmtc_def(apid))
