@@ -1,15 +1,19 @@
+#
+# This file is part of pyspex
+#
+# https://github.com/rmvanhees/pyspex.git
+#
+# Copyright (c) 2019-2022 SRON - Netherlands Institute for Space Research
+#    All Rights Reserved
+#
+# License:  BSD-3-Clause
 """
-This file is part of pyspex
-
-https://github.com/rmvanhees/pyspex.git
-
-Python class to create SPEXone Level-1 products
-
-Copyright (c) 2019-2021 SRON - Netherlands Institute for Space Research
-   All Rights Reserved
-
-License:  BSD-3-Clause
+Contains the classes `L1Aio`, L1Bio and `L1Cio` to write
+PACE/SPEXone data in resp. Level-1A, Level-1B or Level-1C format.
 """
+__all__ = ['L1Aio', 'L1Bio', 'L1Cio', 'write_l1a']
+
+from dataclasses import dataclass
 from datetime import datetime, timedelta, timezone
 from pathlib import Path, PurePosixPath
 
@@ -17,6 +21,10 @@ import numpy as np
 
 from netCDF4 import Dataset
 
+from . import version
+from .hkt_io import read_hkt_nav, write_hkt_nav
+from .lv0_io import (coverage_time, hk_sec_of_day, img_sec_of_day,
+                     nomhk_timestamps, science_timestamps, select_lv0_data)
 from .tm_science import TMscience
 from .lib.attrs_def import attrs_def
 from .lib.l1a_def import init_l1a
@@ -24,33 +32,229 @@ from .lib.l1b_def import init_l1b
 from .lib.l1c_def import init_l1c
 
 # - global parameters -------------------
-MCP_TO_SEC = 1e-7
 
 
+# - local functions ---------------------
 def frac_poly(xx_in, coefs=None):
-    """
-    Temperature [K] calibration derived by Paul Tol (2020-10-21)
+    """Temperature [K] calibration derived by Paul Tol (2020-10-21).
 
     Parameters
     ----------
-    xx    :  ndarray
+    xdata :  ndarray
     coefs :  tuple, default=None
-      coefficients of fractional polynomial: r0, r1, r2, r3, r4
+      Coefficients of fractional polynomial: r0, r1, r2, r3, r4
 
     Returns
     -------
     ndarray, dtype float
     """
-    xx = xx_in.astype(float)
+    xdata = xx_in.astype(float)
 
     if coefs is None:
         coefs = (273.15 + 21.19, 6.97828e+7,
                  -3.53275e-25, 7.79625e-31, -4.6505E-32)
 
     return (coefs[0]
-            + coefs[1] / xx
-            + coefs[2] * xx ** 4
-            + (coefs[3] + coefs[4] * np.log(xx)) * xx ** 5)
+            + coefs[1] / xdata
+            + coefs[2] * xdata ** 4
+            + (coefs[3] + coefs[4] * np.log(xdata)) * xdata ** 5)
+
+
+def get_l1a_name(datatype: str, config: dataclass,
+                 sensing_start: datetime) -> str:
+    """
+    Generate name of Level-1A product based on filename conventions described
+    below
+
+    Parameters
+    ----------
+    datatype :  {'OCAL', 'DARK', 'CAL', 'SCIENCE'}
+       Product datatypes as defined by NASA/GSFC & OB.DAAC.
+    config :  dataclass
+       Settings for the L0->l1A processing.
+    sensing_start :  datetime
+       Start date/time of the first detector frame
+
+    Returns
+    -------
+    str
+        Name of Level-1A product.
+
+    Notes
+    -----
+
+    === Inflight ===
+    L1A file name format, following the NASA ... naming convention:
+       PACE_SPEXONE[_TTT].YYYYMMDDTHHMMSS.L1A[.Vnn].nc
+    where
+       TTT is an optional data type (e.g., for the calibration data files)
+       YYYYMMDDTHHMMSS is time stamp of the first image in the file
+       Vnn file-version number (ommited when nn=1)
+    for example (file-version=1):
+       [Science Product] PACE_SPEXONE.20230115T123456.L1A.nc
+       [Calibration Product] PACE_SPEXONE_CAL.20230115T123456.L1A.nc
+       [Dark science Product] PACE_SPEXONE_DARK.20230115T123456.L1A.nc
+
+    === OCAL ===
+    L1A file name format:
+       SPX1_OCAL_<msm_id>[_YYYYMMDDTHHMMSS]_L1A_vvvvvvv.nc
+    where
+       msm_id is the measurement identifier
+       YYYYMMDDTHHMMSS is time stamp of the first image in the file
+       vvvvvvv is the git-hash string of the pyspex repository
+    """
+    if config.outfile:
+        return config.outfile
+
+    if config.l0_format != 'raw':
+        # inflight product name
+        prod_type = {'DARK': '_DARK',
+                     'CAL': '_CAL',
+                     'OCAL': '_OCAL',
+                     'SCIENCE': ''}.get(datatype.upper(), '')
+        prod_ver = '' if config.file_version == 1\
+            else f'.V{config.file_version:02d}'
+
+        return (f'PACE_SPEXONE{prod_type}'
+                f'.{sensing_start.strftime("%Y%m%dT%H%M%S"):15s}.L1A'
+                f'{prod_ver}.nc')
+
+    # OCAL product name
+    # determine measurement identifier
+    msm_id = config.l0_list[0].stem
+    try:
+        new_date = datetime.strptime(
+            msm_id[-22:], '%y-%j-%H:%M:%S.%f').strftime('%Y%m%dT%H%M%S.%f')
+    except ValueError:
+        pass
+    else:
+        msm_id = msm_id[:-22] + new_date
+
+    return f'SPX1_OCAL_{msm_id}_L1A_{version.get(githash=True)}.nc'
+
+
+def write_lv0_data(prod_name: str, config: dataclass,
+                   science: np.ndarray, nomhk: np.ndarray) -> None:
+    """
+    Write level 0 packages to a level-1A product
+
+    Parameters
+    ----------
+    prod_name :  str
+       Name of the Level-1A product.
+    config :  dataclass
+       Settings for the L0->l1A processing.
+    science : np.ndarray
+       L0 detector data.
+    nomhk : np.ndarray
+       L0 nominal housekeeping data.
+    """
+    # Define data dimensions
+    dims = {'number_of_images': science.size,
+            'samples_per_image': science['hk']['IMRLEN'].max() // 2,
+            'hk_packets': nomhk.size,
+            'SC_records': None}
+
+    # Preprocess the timestamps to be stored in the L1A product
+    # [Science TM]
+    #  - the ICU time stamps are not altered!
+    #  - the variable 'image_time' holds sec_of_day and needs a reference day
+    #    in addition, the data is corrected for the start of integration time.
+    # [nomHK TM]
+    #  - the CCSDS time stamps are not altered!
+    #  - the variable 'nomhk_time' holds sec_of_day and needs a reference day
+    # Only the DSB files use EPOCH 1958 (TAI) all other use EPOCH 1970 (UTC)
+    img_sec, img_subsec = science_timestamps(science)
+    ref_date, img_time = img_sec_of_day(img_sec, img_subsec, science['hk'])
+
+    # Generate and fill L1A product
+    with L1Aio(config.outdir / prod_name, dims=dims,
+               ref_date=ref_date.date()) as l1a:
+        # write image data, detector telemetry and image attributes
+        img_data = np.empty((science.size, dims['samples_per_image']),
+                            dtype='u2')
+        for ii, data in enumerate(science['frame']):
+            img_data[ii, :data.size] = data
+        l1a.fill_science(img_data, science['hk'],
+                         np.bitwise_and(science['hdr']['sequence'], 0x3fff))
+        del img_data
+        l1a.set_dset('/image_attributes/icu_time_sec', img_sec)
+        # modify attribute units for non-DSB products
+        if config.l0_format != 'dsb':
+            l1a.set_attr('valid_min', np.uint32(1577800000),
+                         ds_name='/image_attributes/icu_time_sec')
+            l1a.set_attr('valid_max', np.uint32(1735700000),
+                         ds_name='/image_attributes/icu_time_sec')
+            l1a.set_attr('units', "seconds since 1970-01-01 00:00:00",
+                         ds_name='/image_attributes/icu_time_sec')
+        l1a.set_dset('/image_attributes/icu_time_subsec', img_subsec)
+        l1a.set_dset('/image_attributes/image_time', img_time)
+
+        # write engineering data
+        if nomhk.size > 0:
+            l1a.fill_nomhk(nomhk['hk'])
+            nomhk_sec, nomhk_subsec = nomhk_timestamps(nomhk)
+            hk_time = hk_sec_of_day(nomhk_sec, nomhk_subsec, ref_date)
+            l1a.set_dset('/engineering_data/HK_tlm_time', hk_time)
+
+        # write global attributes
+        if nomhk.size > 0:
+            l1a.set_attr('icu_sw_version', f'0x{nomhk["hk"]["ICUSWVER"][0]:x}')
+        if config.l0_format == 'raw':
+            l1a.fill_global_attrs(inflight=False)
+        else:
+            l1a.fill_global_attrs(inflight=True)
+        l1a.set_attr('input_files', [x.name for x in config.l0_list])
+
+
+# - high-level write function -----------
+def write_l1a(config, science_in, nomhk_in) -> None:
+    """Write Level-1A product.
+
+    Parameters
+    ----------
+    config :  dataclass
+       Settings for the L0->l1A processing.
+    science : np.ndarray
+       L0 detector data.
+    nomhk : np.ndarray
+       L0 nominal housekeeping data.
+    """
+    if config.eclipse is None:
+        # this are "OCAL data" try to write all data to one L1A product.
+        dtype_list = ['OCAL']
+    elif not config.eclipse:
+        # this are "Science data": binned data in "Science mode".
+        dtype_list = ['SCIENCE']
+    else:
+        # this can be "Dark data": binned data using "Science mode" MPSes
+        # and/or "Calibration data”: full frame data in "Diagonstic mode".
+        dtype_list = ['DARK', 'CAL']
+
+    for dtype in dtype_list:
+        # selected L0 data-packages
+        # and group Science packages to detector-frames
+        science, nomhk = select_lv0_data(dtype, science_in, nomhk_in,
+                                         config.verbose)
+        if science.size == 0:
+            continue
+
+        # write L1A product
+        prod_name = get_l1a_name(dtype, config, coverage_time(science)[0])
+        try:
+            write_lv0_data(prod_name, config, science, nomhk)
+        except (PermissionError, RuntimeError) as exc:
+            raise RuntimeError from exc
+
+        # add PACE navigation information from HKT products
+        if config.hkt_list:
+            hkt_nav = read_hkt_nav(config.hkt_list)
+            # select HKT data collocated with Science data
+            # - issue a warning if selection is empty
+            write_hkt_nav(config.outdir / prod_name, hkt_nav)
+
+    if config.verbose:
+        print(f'[INFO]: Successfully generated: {config.outdir / prod_name}')
 
 
 # - class LV1io -------------------------
@@ -58,33 +262,14 @@ class Lv1io:
     """
     Generic class to create SPEXone Level-1 products
 
-    Attributes
+    Parameters
     ----------
-    product: pathlib.Path object
-       Concrete path object to SPEXone Level-1 product
-    fid: netCDF5.Dataset object
-       NetCDF4 Pointer to SPEXone Level-1 product
-    dset_stored: dict
-       Number of items stored for all required netCDF4 variables
-
-    Methods
-    -------
-    close()
-       Close all resources (currently a placeholder function).
-    epoch
-       Provide epoch for SPEXone.
-    get_dim(ds_name)
-       Returns size of dimension.
-    get_attr(attr_name, ds_name=None)
-       Read data of an attribute, global or attached to a group or variable.
-    set_attr(attr_name, value, ds_name=None)
-       Write data to an attribute, global or attached to a group or variable.
-    get_dset(ds_name)
-       Read data of a netCDF4 variable.
-    set_dset(ds_name, value, ibgn=-1)
-       Write/append data to a netCDF4 variable.
-    fill_global_attrs(level, orbit=-1, bin_size=None, inflight=False)
-       Define global attributes in the SPEXone Level-1 products.
+    product :  str
+        name of the SPEXone Level-1 product
+    ref_date :  datetime.date
+    dims :  dict
+    append :  bool, default=False
+        do no clobber, but add new data to existing product
 
     Notes
     -----
@@ -95,44 +280,32 @@ class Lv1io:
     processing_level = 'unknown'
     dset_stored = {}
 
-    def __init__(self, product: str, append=False, **kwargs):
-        """
-        Initialize access to a SPEXone Level-1 product
-
-        Parameters
-        ----------
-        product : str
-           name of the SPEXone Level-1 product
-        append : bool, default=False
-           do no clobber, but add new data to existing product
+    def __init__(self, product: str, ref_date: datetime.date,
+                 dims: dict, append=False):
+        """Initialize access to a SPEXone Level-1 product.
         """
         self.product = Path(product)
         self.fid = None
 
         # initialize private class-attributes
-        self.__epoch = datetime(1970, 1, 1, tzinfo=timezone.utc)
+        self.__epoch = ref_date
 
         # initialize Level-1 product
-        if not append:
-            if self.processing_level == 'L1A':
-                self.fid = init_l1a(product, **kwargs)
-            elif self.processing_level == 'L1B':
-                self.fid = init_l1b(product, **kwargs)
-                if 'ref_date' in kwargs and kwargs['ref_date'] is not None:
-                    self.__epoch = kwargs['ref_date']
-            elif self.processing_level == 'L1C':
-                self.fid = init_l1c(product, **kwargs)
-                if 'ref_date' in kwargs and kwargs['ref_date'] is not None:
-                    self.__epoch = kwargs['ref_date']
-            else:
-                raise KeyError('valid processing levels are: L1A, L1B or L1C')
-        else:
+        if append:
             # open Level-1 product in append mode
             self.fid = Dataset(self.product, "r+")
 
             # store current length of the first dimension
             for key in self.dset_stored:
                 self.dset_stored[key] = self.fid[key].shape[0]
+        elif self.processing_level == 'L1A':
+            self.fid = init_l1a(product, ref_date, dims)
+        elif self.processing_level == 'L1B':
+            self.fid = init_l1b(product, ref_date, dims)
+        elif self.processing_level == 'L1C':
+            self.fid = init_l1c(product, ref_date, dims)
+        else:
+            raise KeyError('valid processing levels are: L1A, L1B or L1C')
 
     def __repr__(self) -> str:
         class_name = type(self).__name__
@@ -144,21 +317,18 @@ class Lv1io:
                 yield attr
 
     def __enter__(self):
-        """
-        method called to initiate the context manager
+        """Method called to initiate the context manager.
         """
         return self
 
     def __exit__(self, exc_type, exc_value, traceback) -> bool:
-        """
-        method called when exiting the context manager
+        """Method called when exiting the context manager.
         """
         self.close()
         return False  # any exception is raised by the with statement.
 
     def close(self) -> None:
-        """
-        Close all resources (currently a placeholder function)
+        """Close all resources (currently a placeholder function).
         """
         if self.fid is None:
             return
@@ -169,21 +339,20 @@ class Lv1io:
     # ---------- PUBLIC FUNCTIONS ----------
     @property
     def epoch(self) -> datetime:
-        """
-        Provide epoch for SPEXone
+        """Provide epoch for SPEXone.
         """
         return self.__epoch
 
     def get_dim(self, name: str):
-        """
-        Get size of a netCDF4 dimension
+        """Get size of a netCDF4 dimension.
         """
         return self.fid.dimensions[name].size
 
     # ----- ATTRIBUTES --------------------
     def get_attr(self, name: str, ds_name=None):
-        """
-        Read data of an attribute, global or attached to a group or variable
+        """Read data of an attribute.
+
+        Global or attached to a group or variable.
 
         Parameters
         ----------
@@ -211,8 +380,9 @@ class Lv1io:
         return res
 
     def set_attr(self, name: str, value, ds_name=None) -> None:
-        """
-        Write data to an attribute, global or attached to a group or variable
+        """Write data to an attribute.
+
+        Global or attached to a group or variable.
 
         Parameters
         ----------
@@ -248,8 +418,7 @@ class Lv1io:
 
     # ----- VARIABLES --------------------
     def get_dset(self, name: str):
-        """
-        Read data of a netCDF4 variable
+        """Read data of a netCDF4 variable.
 
         Parameters
         ----------
@@ -273,8 +442,7 @@ class Lv1io:
         return self.fid[name][:]
 
     def set_dset(self, name: str, value, ibgn=-1) -> None:
-        """
-        Write/append data to a netCDF4 variable
+        """Write/append data to a netCDF4 variable.
 
         Parameters
         ----------
@@ -312,8 +480,7 @@ class Lv1io:
     def fill_global_attrs(self, orbit=-1,
                           bin_size=None,
                           inflight=False) -> None:
-        """
-        Define global attributes in the SPEXone Level-1 products
+        """Define global attributes in the SPEXone Level-1 products.
 
         Parameters
         ----------
@@ -347,53 +514,13 @@ class L1Aio(Lv1io):
     append : boolean, default=False
        If the file is opened in append mode, then parameter 'dims' is ignored
     dims: dictionary, default=None
-       Provide size of various dimensions (L1A only).
-       Default values:
-            number_of_images : None     # number of image frames
-            samples_per_image : 184000  # depends on binning table
-            SC_records : None           # space-craft navigation records
-            hk_packets : None           # number of HK tlm-packets
-            wavelength : None
+       Provide size of various dimensions (L1A only). Default values::
 
-    Attributes
-    ----------
-    product: pathlib.Path object
-       Concrete path object to SPEXone Level-1 product
-    fid: netCDF5.Dataset object
-       NetCDF4 Pointer to SPEXone Level-1 product
-    dset_stored: dict
-       Number of items stored for all required netCDF4 variables
+       number_of_images : None     # number of image frames
+       samples_per_image : 184000  # depends on binning table
+       hk_packets : None           # number of HK tlm-packets
+       wavelength : None
 
-    Methods
-    -------
-    close()
-       Close product and check if required datasets are filled with data.
-    epoch
-       Provide epoch for SPEXone.
-    get_dim(ds_name)
-       Returns size of dimension.
-    get_attr(attr_name, ds_name=None)
-       Read data of an attribute, global or attached to a group or variable.
-    set_attr(attr_name, value, ds_name=None)
-       Write data to an attribute, global or attached to a group or variable.
-    get_dset(ds_name)
-       Read data of a netCDF4 variable.
-    set_dset(ds_name, valu, ibgn=-1e)
-       Write/append data to a netCDF4 variable.
-    fill_global_attrs(level, orbit=-1, bin_size=None, inflight=False)
-       Define global attributes in the SPEXone Level-1 products.
-    check_stored(allow_empty=False)
-       Check variables with the same first dimension have equal sizes.
-    sec_of_day(ccsds_sec, ccsds_subsec)
-       Convert CCSDS timestamp to seconds after midnight.
-    fill_time(ccsds_sec, ccsds_subsec, group=None)
-       Write time of Science telemetry packets (UTC/TAI) to L1A product.
-    fill_science(img_data, img_hk, img_id)
-       Write Science data and housekeeping telemetry (Science) to L1A product.
-    fill_nomhk(nomhk_data)
-       Write nominal housekeeping telemetry packets (NomHK) to L1A product.
-    fill_demhk(demhk_data)
-       Write detector housekeeping telemetry packets (DemHK) to L1A product.
     """
     processing_level = 'L1A'
     dset_stored = {
@@ -403,27 +530,20 @@ class L1Aio(Lv1io):
         '/image_attributes/digital_offset': 0,
         '/image_attributes/nr_coadditions': 0,
         '/image_attributes/exposure_time': 0,
-        '/image_attributes/image_CCSDS_sec': 0,
-        '/image_attributes/image_CCSDS_subsec': 0,
+        '/image_attributes/icu_time_sec': 0,
+        '/image_attributes/icu_time_subsec': 0,
         '/image_attributes/image_time': 0,
         '/image_attributes/image_ID': 0,
         '/engineering_data/NomHK_telemetry': 0,
-        '/engineering_data/DemHK_telemetry': 0,
+        # '/engineering_data/DemHK_telemetry': 0,
         '/engineering_data/temp_detector': 0,
         '/engineering_data/temp_housing': 0,
         '/engineering_data/temp_radiator': 0,
-        '/engineering_data/HK_tlm_time': 0,
-        '/navigation_data/adstate': 0,
-        '/navigation_data/att_quat': 0,
-        '/navigation_data/orb_pos': 0,
-        '/navigation_data/orb_vel': 0,
-        '/navigation_data/att_time': 0,
-        '/navigation_data/orb_time': 0
+        '/engineering_data/HK_tlm_time': 0
     }
 
     def close(self):
-        """
-        Close product and check if required datasets are filled with data
+        """Close product and check if required datasets are filled with data.
         """
         if self.fid is None:
             return
@@ -437,36 +557,25 @@ class L1Aio(Lv1io):
         # check of all required dataset their sizes
         self.check_stored(allow_empty=True)
 
-        # define object to access Science telemetry parameters
-        mps = TMscience(self.get_dset('/science_data/detector_telemetry')[-1])
+        # determine time_coverage_start
+        dset = self.fid['/image_attributes/image_time']
+        tstamp = datetime(int(dset.year), int(dset.month), int(dset.day),
+                          tzinfo=timezone.utc)
+        tstamp += timedelta(seconds=float(dset[0].data))
+        self.fid.time_coverage_start = tstamp.isoformat(timespec='milliseconds')
 
-        # determine duration master clock cycle
-        imro = 1e-1 * mps.get('FTI') * 2
-        mcycl = 1e-1 * mps.get('FTI') * mps.get('REG_NCOADDFRAMES')
-
-        img_sec = self.fid['/image_attributes/image_CCSDS_sec'][:].data
-        img_subsec = self.fid['/image_attributes/image_CCSDS_subsec'][:].data
-
-        time0 = (self.epoch
-                 + timedelta(seconds=int(img_sec[0]))
-                 + timedelta(microseconds=int(1e6 * img_subsec[0] / 65536))
-                 - timedelta(milliseconds=mcycl + imro))
-
-        time1 = (self.epoch
-                 + timedelta(seconds=int(img_sec[-1]))
-                 + timedelta(microseconds=int(1e6 * img_subsec[-1] / 65536))
-                 - timedelta(milliseconds=imro))
-
-        self.fid.time_coverage_start = time0.isoformat(timespec='milliseconds')
-        self.fid.time_coverage_end = time1.isoformat(timespec='milliseconds')
+        # determine time_coverage_end
+        tstamp = datetime(int(dset.year), int(dset.month), int(dset.day),
+                          tzinfo=timezone.utc)
+        tstamp += timedelta(seconds=float(dset[-1].data))
+        self.fid.time_coverage_end = tstamp.isoformat(timespec='milliseconds')
 
         self.fid.close()
         self.fid = None
 
     # -------------------------
     def check_stored(self, allow_empty=False):
-        """
-        Check variables with the same first dimension have equal sizes
+        """Check variables with the same first dimension have equal sizes.
 
         Parameters
         ----------
@@ -506,96 +615,9 @@ class L1Aio(Lv1io):
         for ii in indx:
             print(warn_str.format(key_list[ii], res[ii]))
 
-        # check navigation datasets
-        dim_sz = self.get_dim('SC_records')
-        key_list = [x for x in self.dset_stored
-                    if x.startswith('/navigation_data')]
-        res = []
-        for key in key_list:
-            res.append(self.dset_stored[key])
-        res = np.array(res)
-        if allow_empty:
-            indx = ((res > 0) & (res != dim_sz)).nonzero()[0]
-        else:
-            indx = (res != dim_sz).nonzero()[0]
-        for ii in indx:
-            print(warn_str.format(key_list[ii], res[ii]))
-
     # ---------- PUBLIC FUNCTIONS ----------
-    def sec_of_day(self, ccsds_sec, ccsds_subsec) -> tuple:
-        """
-        Convert CCSDS timestamp to seconds after midnight
-
-        Parameters
-        ----------
-        ccsds_sec : numpy array (dtype='u4')
-          Seconds since 1970-1-1
-        ccsds_subsec : numpy array (dtype='u2')
-          Sub-seconds as (1 / 2**16) seconds
-
-        Returns
-        -------
-        tuple holding reference_day and numpy.ndarray with sec_of_day
-        """
-        # determine midnight before start measurement
-        tstamp0 = self.epoch + timedelta(seconds=int(ccsds_sec[0]))
-        reference_day = datetime(year=tstamp0.year,
-                                 month=tstamp0.month,
-                                 day=tstamp0.day, tzinfo=timezone.utc)
-
-        # store seconds since midnight
-        sec_of_day = ccsds_sec - (reference_day - self.epoch).total_seconds()
-
-        # return seconds since midnight
-        return reference_day, sec_of_day + ccsds_subsec / 65536
-
-    def fill_time(self, ccsds_sec, ccsds_subsec, group=None) -> None:
-        """
-        Write time of Science telemetry packets (UTC/TAI) to L1A product
-
-        Parameters
-        ----------
-        ccsds_sec : numpy array (dtype='u4')
-          Seconds since 1970-1-1
-        ccsds_subsec : numpy array (dtype='u2')
-          Sub-seconds as (1 / 2**16) seconds
-        group : str, default=None
-
-        Note
-        ----
-        Writes parameters: image_time, image_CCSDS_sec and image_CCSDS_subsec
-        """
-        if group is None:
-            group = 'image_attributes'
-
-        # calculate seconds of day
-        reference_day, sec_of_day = self.sec_of_day(ccsds_sec, ccsds_subsec)
-
-        if group in ('image_attributes', '/image_attributes'):
-            self.set_dset('/image_attributes/image_CCSDS_sec', ccsds_sec)
-            self.set_dset('/image_attributes/image_CCSDS_subsec', ccsds_subsec)
-            self.set_dset('/image_attributes/image_time', sec_of_day)
-            self.set_attr('units',
-                          f'seconds since {reference_day.isoformat()}',
-                          ds_name='/image_attributes/image_time')
-        elif group in ('engineering_data', '/engineering_data'):
-            self.set_dset('/engineering_data/HK_tlm_time', sec_of_day)
-            self.set_attr('units',
-                          f'seconds since {reference_day.isoformat()}',
-                          ds_name='/engineering_data/HK_tlm_time')
-        else:
-            self.set_dset('/navigation_data/att_time', sec_of_day)
-            self.set_attr('units',
-                          f'seconds since {reference_day.isoformat()}',
-                          ds_name='/navigation_data/att_time')
-            self.set_dset('/navigation_data/orb_time', sec_of_day)
-            self.set_attr('units',
-                          f'seconds since {reference_day.isoformat()}',
-                          ds_name='/navigation_data/orb_time')
-
     def fill_science(self, img_data, img_hk, img_id) -> None:
-        """
-        Write Science data and housekeeping telemetry (Science) to L1A product
+        """Write Science data and housekeeping telemetry (Science).
 
         Parameters
         ----------
@@ -604,14 +626,14 @@ class L1Aio(Lv1io):
         img_hk : numpy array ()
            Structured array with all Science telemetry parameters
         img_id : numpy array (uint16)
-           Detector image counter
+           Detector frame counter modulo 0x3FFF
 
         Notes
         -----
         Adds detector_telemetry data to the group /science_data
 
         Parameters: binning_table, digital_offset, exposure_time
-        and nr_coadding are extracted from the telemetry packets and writen
+        and nr_coadditions are extracted from the telemetry packets and writen
         in the group /image_attributes
         """
         if len(img_hk) == 0:
@@ -619,19 +641,16 @@ class L1Aio(Lv1io):
 
         self.set_dset('/science_data/detector_images', img_data)
         self.set_dset('/science_data/detector_telemetry', img_hk)
-
-        mps = TMscience(img_hk)
-        self.set_dset('/image_attributes/binning_table', mps.binning_table_id)
-        self.set_dset('/image_attributes/digital_offset', mps.offset)
-        self.set_dset('/image_attributes/exposure_time',
-                      MCP_TO_SEC * mps.exp_time)
-        self.set_dset('/image_attributes/nr_coadditions',
-                      mps.get('REG_NCOADDFRAMES'))
         self.set_dset('/image_attributes/image_ID', img_id)
 
+        tm_sc = TMscience(img_hk)
+        self.set_dset('/image_attributes/binning_table', tm_sc.binning_table)
+        self.set_dset('/image_attributes/digital_offset', tm_sc.digital_offset)
+        self.set_dset('/image_attributes/exposure_time', tm_sc.exposure_time)
+        self.set_dset('/image_attributes/nr_coadditions', tm_sc.nr_coadditions)
+
     def fill_nomhk(self, nomhk_data):
-        """
-        Write nominal house-keeping telemetry packets (NomHK) to L1A product
+        """Write nominal house-keeping telemetry packets (NomHK).
 
         Parameters
         ----------
@@ -672,8 +691,7 @@ class L1Aio(Lv1io):
                           frac_poly(nomhk_data['TS3_RADIATOR_N_T']))
 
     def fill_demhk(self, demhk_data):
-        """
-        Write detector housekeeping telemetry packets (DemHK) to L1A product
+        """Write detector housekeeping telemetry packets (DemHK).
 
         Parameters
         ----------
@@ -695,8 +713,7 @@ class L1Aio(Lv1io):
 
 # - class L1Bio -------------------------
 class L1Bio(Lv1io):
-    """
-    This class can be used to create a SPEXone Level-1B product
+    """This class can be used to create a SPEXone Level-1B product
 
     Parameters
     ----------
@@ -710,36 +727,6 @@ class L1Bio(Lv1io):
     spatial_samples: int, default=200
        Total number of spatial samples from all viewport
 
-    Attributes
-    ----------
-    product: pathlib.Path object
-       Concrete path object to SPEXone Level-1 product
-    fid: netCDF5.Dataset object
-       NetCDF4 Pointer to SPEXone Level-1 product
-    dset_stored: dict
-       Number of items stored for all required netCDF4 variables
-
-    Methods
-    -------
-    close()
-       Close product and check if required datasets are filled with data.
-    epoch
-       Provide epoch for SPEXone.
-    get_dim(ds_name)
-       Returns size of dimension.
-    get_attr(attr_name, ds_name=None)
-       Read data of an attribute, global or attached to a group or variable.
-    set_attr(attr_name, value, ds_name=None)
-       Write data to an attribute, global or attached to a group or variable.
-    get_dset(ds_name)
-       Read data of a netCDF4 variable.
-    set_dset(ds_name, value, ibgn=-1)
-       Write/append data to a netCDF4 variable.
-    fill_global_attrs(level, orbit=-1, bin_size=None)
-       Define global attributes in the SPEXone Level-1 products.
-    check_stored()
-       Check variables with the same first dimension have equal sizes.
-
     Notes
     -----
     ToDo: make sure we store the reference date for image_time
@@ -775,8 +762,7 @@ class L1Bio(Lv1io):
     }
 
     def close(self):
-        """
-        Close product and check if required datasets are filled with data
+        """Close product and check if required datasets are filled with data.
         """
         if self.fid is None:
             return
@@ -809,8 +795,7 @@ class L1Bio(Lv1io):
 
     # -------------------------
     def check_stored(self):
-        """
-        Check variables with the same first dimension have equal sizes
+        """Check variables with the same first dimension have equal sizes.
         """
         warn_str = ('SPEX Level-1B format check [WARNING]:'
                     ' size of variable "{:s}" is wrong, only {:d} elements')
@@ -849,8 +834,7 @@ class L1Bio(Lv1io):
 
 # - class L1Cio -------------------------
 class L1Cio(Lv1io):
-    """
-    This class can be used to create a SPEXone Level-1C product
+    """This class can be used to create a SPEXone Level-1C product.
 
     Parameters
     ----------
@@ -862,38 +846,6 @@ class L1Cio(Lv1io):
        Number of images used as input to generate the L1B product.
        This dimension is by default UNLIMITED.
 
-    Attributes
-    ----------
-    product: pathlib.Path object
-       Concrete path object to SPEXone Level-1 product
-    inflight: boolean
-       Flag to indicate data collected during in-flight of on-ground
-    fid: netCDF5.Dataset object
-       NetCDF4 Pointer to SPEXone Level-1 product
-    dset_stored: dict
-       Number of items stored for all required netCDF4 variables
-
-    Methods
-    -------
-    close()
-       Close product and check if required datasets are filled with data.
-    epoch
-       Provide epoch for SPEXone.
-    get_dim(ds_name)
-       Returns size of dimension.
-    get_attr(attr_name, ds_name=None)
-       Read data of an attribute, global or attached to a group or variable.
-    set_attr(attr_name, value, ds_name=None)
-       Write data to an attribute, global or attached to a group or variable.
-    get_dset(ds_name)
-       Read data of a netCDF4 variable.
-    set_dset(ds_name, value)
-       Write/append data to a netCDF4 variable.
-    fill_global_attrs(level, orbit=-1, bin_size=None)
-       Define global attributes in the SPEXone Level-1 products.
-    check_stored()
-       Check variables with the same first dimension have equal sizes.
-
     Notes
     -----
     ToDo: make sure we store the reference date for image_time
@@ -937,8 +889,7 @@ class L1Cio(Lv1io):
     }
 
     def close(self):
-        """
-        Close product and check if required datasets are filled with data
+        """Close product and check if required datasets are filled with data.
         """
         if self.fid is None:
             return
@@ -971,8 +922,7 @@ class L1Cio(Lv1io):
 
     # -------------------------
     def check_stored(self):
-        """
-        Check variables with the same first dimension have equal sizes
+        """Check variables with the same first dimension have equal sizes.
         """
         warn_str = ('SPEX Level-1C format check [WARNING]:'
                     ' size of variable "{:s}" is wrong, only {:d} elements')
